#!/usr/bin/perl -w

# graphs.cgi - The Frontend of the nagios grapher
# Copyright (C) 2004, NETWAYS GmbH, Gerd Mueller, Marius Hein
# $Id: graphs.cgi 1028 2005-11-30 10:53:04Z mhein $
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# Main
use strict;
use CGI qw(:standard);
use CGI::Carp qw(fatalsToBrowser);
use NagiosGrapher;
use vars qw (
    $self
    $stylesheet
    $q
    %params
    $height
    $width
    $graph_script
    $host
    $old_host
    $service
    $old_service
    @rrd
    %rrd_desc
    %rrd_cmd
    $title
    $refresh
    %refresh_values
    $page
    %page_hash
    $page_act
    $img_source
    %type_values
    $type
    $use_browser
    $use_browser_overwrite
    $xmlhash
    $action
    $ng
);

$| = 1;

$ng = NagiosGrapher::getInstance();

# The SUBS:
sub get_pages;
sub get_stylesheet;
sub action_graph;
sub action_rrdinfo;
sub action_showtemplate;

$graph_script = "./rrd2-system.cgi";

$q = new CGI;

# Self referencing url
$self = $q->self_url;
$self =~ s/;/&/g; # making selfreferencing URL without ';' for meta reload tag.

# The param hash from the CGI Object
%params = $q->Vars;

# Refresh param (seconds)
$refresh=$params{refresh};
$refresh=300 if (!defined($refresh));

# Graph type param (string)
$type=$params{type};
$type="AVERAGE" unless(defined($type));

# Graph height param (integer)
$height=$params{height};
$height=250 if (!defined($height) || length($height) <= 0);

# Graph width param (integer)
$width=$params{width};
$width=595 if (!defined($width) || length($width) <= 0);

# The major params ... (srtings)
$host=$params{host};
$service=$params{service};

# The params for the hiddenfields (strings)
$old_host = $params{old_host};
$old_service = $params{old_service};

# Action
$action = uc($params{action});
$action = 'GRAPH' if (!defined($action) || length($action) <= 0);

# SiteTitle
$title = "graph analysis for $host, $service";

$xmlhash = $ng->get_ngrapherxml;
# If the current differs from the selected, start the default service ...
if (defined($old_host) && $old_host ne $host) {
    $service = [sort(keys(%{$xmlhash->{host}->{$host}->{service}}))]->[0];
    $q->param("service", $service);
}

# Starting the HTML Output ...

# DTD and default HTTP header
$q->default_dtd('-//W3C//DTD HTML 4.01 Transitional//EN');
print $q->header(-type=>'text/html', -expires=>'+1s');

# HTML Header, with / without refresh ...
if ($refresh && $action eq 'GRAPH') {
    print $q->start_html(
                            -lang=>'de-de',
                            -title=>$title,
                            -style=>{
                                     -code=>get_stylesheet()
                                    },
                            -head=>meta({
                                    -http_equiv => "refresh",
                                    -content => $refresh. "; URL=". $self,
                                   })
                        );
} else {
    print $q->start_html(
                         -lang=>'de-de',
                         -title=>$title,
                         -style=>{
                                  -code=>get_stylesheet()
                                 },
                     );
}

# ---
# The ACTIONS
# ---
unless (defined($ng)) {
	print '<h4>collect2.pl is not running ...</h4>';
	print '<p>The NagiosGrapher Object Cache was not found. Please make shure that the collect2.pl daemon is running. (<a href="'. $self. '">reload</a>)</p>';
}
else {
	if ($action eq 'GRAPH') {
		action_graph();
	}
	elsif($action eq 'RRDINFO') {
		action_rrdinfo();
	}
	elsif($action eq 'SHOWTEMPLATE') {
		action_showtemplate();
	}
	else {
		print '<h4>No action found ('. $action. ') ...</h4>';
	}
}


# End HTML Output ...
print $q->end_html();

# Generic END
exit(0);

sub action_showtemplate {
	# The maintable
	print "<table border=\"1\" cellpadding=\"4\" cellspacing=\"0\" class=\"table1\" align=\"left\">\n";
	print '<tr><td>'. "\n";
	
	# The MENU:
	print $q->span({-class => "head1"}, "NETWAYS Nagios Grapher v$NagiosGrapher::VERSION"), "&nbsp;(Logged in as $ENV{REMOTE_USER})";
	print "<hr size=\"1\" />";
	
	print $q->span({-class => "head2"}, "Graphtemplate for <code>$service</code>:"). "&nbsp;";
	print '<br />'. $q->a({-href=>'Javascript: history.back();'}, 'Go back');
	print '<br /><br />';
	
	my @blocks = $ng->get_ngrapherdef($service);
	if (@blocks > 0) {
	print '<table>';
	foreach (@blocks) {
		
		print '<tr><td colspan="3"><b>define {</b></td></tr>';
		
		foreach my $template_key (sort(keys(%{ $_ }))) {
			next if ($template_key eq 'define');
			print '<tr>';
			
			print '<td width="30">&nbsp;</td>';
			print '<td width="120">'. $template_key. '</td>';
			print '<td>'. $_->{$template_key}. '</td>';
			
			print '</tr>';
		}
		
		print '<tr><td colspan="3"><b>}</b></td></tr>';
		print '<tr><td colspan="3">&nbsp;</td></tr>';
		
	}
	print '</table>';
	}
	else {
		print '<br /><b>No templatedata found !</b>';
	}
	print '</td></tr></table>';
}

sub action_rrdinfo {
	# The maintable
	print "<table border=\"1\" cellpadding=\"4\" cellspacing=\"0\" class=\"table1\" align=\"left\">\n";
	print '<tr><td>'. "\n";
	
	# The MENU:
	print $q->span({-class => "head1"}, "NETWAYS Nagios Grapher v$NagiosGrapher::VERSION"), "&nbsp;(Logged in as $ENV{REMOTE_USER})";
	print "<hr size=\"1\" />";
	
	print $q->span({-class => "head2"}, "RRDInfo for <code>$host</code>, <code>$service</code>:"). "&nbsp;";
	print '<br />'. $q->a({-href=>'Javascript: history.back();'}, 'Go back');
	print '<br /><br />';
	
	$xmlhash = $ng->get_ngrapherxml;
	if (exists($xmlhash->{host}->{$host}->{service}->{$service}->{rrd_file})) {
		my $info = $ng->rrd_info($host, $xmlhash->{host}->{$host}->{service}->{$service}->{rrd_file});
		print '<table>';
		my $i=0;
		foreach (sort(keys(%{ $info }))) {
			$i++;
			if ($i%2) {
				print '<tr>';
			}
			else {
				print '<tr bgcolor="#f1f1f1">'
			}
			print '<td><b>'. $_. ':</b></td>';
			print '<td>'. ($info->{$_} || '<small style="color: #c0c0c0;">(empty)</small>'). '</td>';
			print '</tr>';
		}
		print '</table>';
	}
	else {
		print '<br /><b>No RRDFile found !</b>';
	}
	
	print '</td></tr></table>';
}

sub action_graph {
	# Some fixed values for combo boxes
	%refresh_values = (
	                    ""     => "None",
	                    "30"    => "30 seconds",
	                    "60"    => "1 minute",
	                    "300"   => "5 minutes",
	                    "600"   => "10 minutes",
	                  );
	
	%type_values = (
	                    AVERAGE => "Average values",
	                    MAX => "Max values",
	                    MIN => "Min values",
	               );
	
	# XML HASH
	$xmlhash = $ng->get_ngrapherxml;
	
	# Init the NMgraphs
	my @nmgraphs = $ng->get_nmgrapherdef();
	foreach (@nmgraphs) {
	    unless ($xmlhash->{host}->{$_->{host_name}}) {
	        $xmlhash->{host}->{$_->{host_name}}->{service}->{$_->{service_name}} = {};
	    }
	    else {
	        unless ($xmlhash->{host}->{$_->{host_name}}->{service}->{$_->{service_name}}) {
	            $xmlhash->{host}->{$_->{host_name}}->{service}->{$_->{service_name}} = {};
	        }
	    }
	}
	
	# Use the host/service browser (COMMON) or not
	$use_browser=$ng->get_ngraphercfg_value('fe_use_browser_all');
	$use_browser_overwrite = $params{browser};
	
	# Overwrite the browser through the URL
	$use_browser=$use_browser_overwrite if (defined($use_browser_overwrite) && $ng->get_ngraphercfg_value('fe_use_browser_url') eq "1");
	
	# Check if the user can view the browser ...
	if ( (defined($use_browser) && $use_browser eq "1")
	    || ($ng->AuthCheckType($ENV{REMOTE_USER}, 'fe_use_browser_for')) ) {
	    $use_browser=1;
	    $q->param("old_host", $host);
	    $q->param("old_service", $service);
	}
	
	# If the current differs from the selected, start the default service ...
	if (defined($old_host) && $old_host ne $host) {
	    $service = [sort(keys(%{$xmlhash->{host}->{$host}->{service}}))]->[0];
	    $q->param("service", $service);
	}
	
	# Check if it is a multigraph
	my $multigraph = 0;
	unless(exists($xmlhash->{host}->{$host}->{service}->{$service}->{rrd_file})) {
		$multigraph = 1;
	}
	
	# Preparing the page browsing
	%page_hash = get_pages();
	if (keys(%page_hash)>1) {
	    $page=1;
	    $page_act=$params{act_page};
	    $page_act=[keys(%page_hash)]->[1] unless (defined $page_act);
	    $page_act=undef if ($page_act eq '-ALL');
	}
	
	# RRD Type Array
	@rrd = (0..4);
	@rrd = sort(@rrd);
	# RRD Description HASH
	%rrd_desc = (0 => "Actual", 1 => "Daily", 2 => "Weekly", 3 => "Monthly", 4 => "Yearly");
	# RRD TimeInterval HASH
	%rrd_cmd = (0 => 5400, 1 => 129600, 2 => 907200, 3 => 4017600, 4 => 48211200);
	
	# The maintable
	print "<table border=\"1\" cellpadding=\"4\" cellspacing=\"0\" class=\"table1\" align=\"left\">\n";
	print '<tr><td colspan="2">'. "\n";
	
	# The MENU:
	print $q->span({-class => "head1"}, "NETWAYS Nagios Grapher v$NagiosGrapher::VERSION"), "&nbsp;(Logged in as $ENV{REMOTE_USER})";
	print "<hr size=\"1\" />";
	
	print '<table border="0" cellpadding="0" cellspacing="0" style="width: 100%;"><tr><td>';
	# Graph description
	print $q->span({-class => "head2"}, "Graphs for <code>$host</code>, <code>$service</code>:"). "&nbsp;";
	
	# The image anchors
	foreach (@rrd) {
	    print "[&nbsp;",
	        $q->a({href=>$q->self_url. "#". $rrd_desc{$_}}, $rrd_desc{$_}),
	        "&nbsp;]&nbsp;",
	        "\n";
	}
	
	# Close and reload link
	print
	    '</td><td align="right">',
	    "[&nbsp;", $q->a({-href=>"#", -onclick=>"window.location.reload(); return true;"}, "Reload"), "&nbsp;]",
	    "&nbsp;&nbsp;[&nbsp;", $q->a({-href=>"#", -onclick=>"top.close(); return true;"}, "Close"), "&nbsp;]",
	    '</td></tr></table>';
	
	# Starting the form
	print
	    "\n\n",
	    $q->start_form(-method=>"get", -action=>$q->self_url, -enctype=>"application/x-www-form-urlencoded");
	    # Writing some hiddenfields, with or without browser ...
	    if (defined($use_browser) && $use_browser eq "1") {
	        print
	            $q->hidden(-name=>"old_host", -default=>$host),
	            $q->hidden(-name=>"old_service", -default=>$service);
	        # Remind the overwriting setting if allowed ...
	        print
	            $q->hidden(-name=>"browser", -default=>$use_browser_overwrite)
	            if (defined($use_browser_overwrite) && $use_browser_overwrite eq "1");
	    }
	    else {
	        print
	            $q->hidden(-name=>"host", -default=>$host),
	            $q->hidden(-name=>"service", -default=>$service);
	    }
	    
	print
	    # Starting the table for from elements
	    '<table border="0" cellpadding="2" cellspacing="0">',
	    '<tr>',
	    
	    # Width field
	    '<td>Width:</td><td>',
	    $q->textfield(-name=>"width", -default=>$width, -override=>1, -size=>4), '</td>',
	    
	    # Height field
	    '<td>Height:</td><td>',
	    $q->textfield(-name=>"height", -default=>$height, -override=>1, -size=>4), '</td>',
	    
	    # Refresh Combo
	    '<td>Refresh</td><td>',
	    $q->popup_menu(
	                    -name=>"refresh",
	                    -default=>$refresh,
	                    -values=>[keys(%refresh_values)],
	                    -labels=>\%refresh_values,
	                    -onchange=>"this.form.submit();",
	                      ), '</td></tr>';
	    
	    # Displaying the page combo if needed...
	    if ($page) {
	    print "<tr><td>page:</td><td colspan=\"5\">", $q->popup_menu(
	                         -name=>"act_page",
	                         -default=>$page_act,
	                         -values=>[keys(%page_hash)],
	                         -labels=>\%page_hash,
	                         -style=>"width: 287px; background-color: #fc8888;",
	                         -onchange=>"this.form.submit();",
	                         ), '</td></tr>';
	    }
	    
	    # Type combo
	    print
	    '<tr><td>Type:</td><td colspan="5">',
	    $q->popup_menu(
	                    -name=>"type",
	                    -default=>$type,
	                    -values=>[keys(%type_values)],
	                    -labels=>\%type_values,
	                    -style=>"width: 287px; background-color: #bbccff;",
	                    -onchange=>"this.form.submit();",
	                  ), '</td></tr>';
	    
	    # Only use the browser if it ist allowed ;-) ...
	    # Can be overwritten if allowed ...
	    # Overwrite handling in top of the script.
	    if (defined($use_browser) && $use_browser eq "1") {
	        # Host combo
	        print
	        '<tr><td>Host:</td><td colspan="5">',
	        $q->popup_menu(
	                       -name=>"host",
	                       -default=>$host,
	                       -values=>[sort(keys(%{$xmlhash->{host}}))],
	                       -style=>"width: 287px; background-color: #ffff00;",
	                       -onchange=>"this.form.submit();",
	                      ), '</td></tr>';
	        
	        # Service combo
	        print
	        '<tr><td>Service:</td><td colspan="5">',
	        $q->popup_menu(
	                       -name=>"service",
	                       -default=>$service,
	                       -values=>[sort(keys(%{$xmlhash->{host}->{$host}->{service}}))],
	                       -style=>"width: 287px; background-color: #ffff00;",
	                       -onchange=>"this.form.submit();",
	                      ), '</td></tr>';
	    }
	    
	    print
	    '<tr>
	    	<td>&nbsp;</td>
	    	<td colspan="4">'. ($multigraph == 1 ? '&nbsp;' : ('[&nbsp;'. $q->a({-href=>$self. '&action=rrdinfo'}, 'RRDInfo'). '&nbsp;]&nbsp;&nbsp;[&nbsp;'. $q->a({-href=>$self. '&action=showtemplate'}, 'TemplateInfo'). '&nbsp;]')). '</td>
	    	<td align="right">', $q->submit(-name=>"cmd_change", -value=>"Change"), '</td></tr>',
	    
	    '</table>',
	    $q->endform,
	    "\n\n";
	    
	print '<table border="0" cellspacing="0">'. "\n";
	
	# Displaying the RRD's
	foreach (@rrd) {
	    #print "<tr><td valign=\"top\">". $rrd_desc{$_}. " graph:",
	    #    $q->br,
	    #    "[&nbsp;", $q->a({-href=>$q->self_url. "#"}, "To top"), "&nbsp;]",
	    #    "</td><td>\n";
	    
	    $img_source = $graph_script. '?host='. $host. ';service='. $service. ';start=-'. $rrd_cmd{$_}. ';title='. $rrd_desc{$_}. ';width='. $width. ';height='. $height;
	    $img_source .= ';act_page='. $page_act if (defined($page_act));
	    $img_source .= ';type='. $type if (defined($type));
	    
	    # Print the menu
	    print "<tr><td width=\"45\">&nbsp;</td><td>";
	    print '<table class="rrdheader"><tr>';
	    print '<td>'. '[&nbsp;', $q->a({name=>$rrd_desc{$_}, -href=>$q->self_url. '#'}, 'To top'), '&nbsp;]&nbsp;'. $rrd_desc{$_}. ' graph'. '</td>';
	    print '<td align="right">View as: ',
	    	 $q->a({-href=>$img_source. ';imageformat=pdf', -target=>'_blank'}, 'PDF'), ', ',
	    	 $q->a({-href=>$img_source. ';imageformat=svg', -target=>'_blank'}, 'SVG'), ', ',
	    	 $q->a({-href=>$img_source. ';imageformat=eps', -target=>'_blank'}, 'EPS'),
	    	 '</td>';
	    print '</tr></table>';
	    
	    # Print the RRD Image
	    print $q->a({-href => $img_source, -target=>"_blank"},
	    	$q->img({src => $img_source, border => 0, alt => $rrd_desc{$_}}));
	    print "</td></tr>\n";
	    
	    # Some space at the end of a image
	    print '<tr><td colspan="2">&nbsp;</td></tr>';
	}
	
	# Footer
	print
	    '<tr><td colspan="2" align="right">',
	    'Nagios Grapher v'. $NagiosGrapher::VERSION. '&nbsp;&copy;&nbsp;<span style="font-style: bolder">MMIV, MMV</span>&nbsp;(2004, 2005)&nbsp;',
	    $q->a({-href=>"http://www.netways.de/", -target=>"_blank"}, "NETWAYS GmbH"),
	    '</td></tr>';
	
	print "</table></td></tr></table>\n";
}

sub get_stylesheet {
	# Default Stylesheet
	my $css =<<ECSS;
	    body { font-family: 'Arial'; font-size: 8pt; color: #666666; background-color: #f1f1f1; }
	    
	    td { font-size: 8pt; color: #333333; }
	    
	    a { font-size: 8pt; text-decoration: none; color: #4d8fd2; font-weight: normal; }
	    
	    a:hover { text-decoration: underline; }
	    
	    .head2 { color: #333333; font-size: 8pt; font-weight: normal; }
	    
	    .head2 code { color: #cc0000; font-size: 8pt; font-weight: normal; }
	    
	    .head1 { color: #333333; font-size: 12pt; font-weight: bolder; }
	    
	    .table1 {
	        margin-left: 10px;
	        background-color: #ffffff;
	        border-collapse: collapse;
	        border-color: #f1f1f1;
	    }
	    
	    .table1 td {
	        border-color: #f1f1f1;
	    }
	    
	    table.rrdheader {
	    	width: 100%;
	    	background-color: #f1f1f1;
	    	border-left: 2px #c0c0c0 solid;
	    	border-right: 2px #c0c0c0 solid;
	    	border-top: 2px #c0c0c0 solid;
	    }
	    
	    table.rrdheader td { padding: 2px; }
	    
	    div.line { height: 1px; font-size: 1px; background-color: #000080; margin: 5px 0px 5px 0px; }
ECSS
	
	return $css;
}

# Sub get_pages: extract the pagebrowser values from configuration
sub get_pages {
    my (%out);
    my @conf = $ng->get_ngrapherdef($service, "GRAPH");
    foreach (@conf) {
        if (exists $_->{page}) {
            $out{$_->{page}} = $_->{page}
        }
    }
    $out{"-ALL"} = '--ALL--';
    return %out;
}

